@@ -1,3 +1,9 @@
+/*!
+# Desktop Entry
+
+This is the rustification of the desktop entry specifications!
+*/
+
 // desktop_entry.rs
 // Rusified in 2021 Copyright Israel Dahl. All rights reserved.
 // 
@@ -24,7 +30,7 @@
 extern crate tini;
 use tini::Ini;
 use std::fmt;
-/// could just say use crate::utils::*
+// could just say use crate::utils::*
 use crate::utils::get_language;
 use crate::utils::to_bool;
 
@@ -72,11 +78,13 @@ pub enum DesktopEnvironment {
 }
 impl DesktopEnvironment {
     #[allow(dead_code)]
+    /// This is to allow people to iterate over the `enum` nicely
     pub fn iter() -> Iter<'static, DesktopEnvironment> {
         static ONLYSHOWIN:[DesktopEnvironment; 17] = [GNOME, GNOMEClassic, GNOMEFlashback, KDE, LXDE, LXQt, MATE, Razor, ROX, TDE, Unity, XFCE, EDE, Cinnamon, Pantheon, Old,  Unknown];
         ONLYSHOWIN.iter()
     }
     #[allow(dead_code)]
+    /// Convert the DE to a `String`
     pub fn to_string(&self) ->String {
       match &self {
           GNOME => "GNOME".to_string(),
@@ -99,6 +107,7 @@ impl DesktopEnvironment {
       }
     }
     #[allow(dead_code)]
+    /// Take a String and return a `DesktopEnvironment`
     pub fn from_string(item:String) -> DesktopEnvironment {
         if item == "GNOME" {
             return DesktopEnvironment::GNOME
@@ -150,12 +159,37 @@ impl fmt::Display for DesktopEnvironment {
 /// To allow the addition of new types in the future, implementations should ignore desktop entries with an "unknown" type.
 #[derive(Debug)]
 pub enum DesktopType {
+    /// A program
     Application,
+    /// A website
     Link,
+    /// A file system directory
     Directory,
     /// Ignore 'Unknown' type in your program, it really just means it is anything other than the above known types
     Unknown,
 }
+impl DesktopType {
+    /// Convert a `String` into a `DesktopType`
+    pub fn from_string(dt:String)->DesktopType {
+        if dt == "Application" {
+            return DesktopType::Application
+        } else if dt == "Link" {
+            return DesktopType::Link
+        } else if dt == "Directory" {
+            return DesktopType::Directory
+        }
+        DesktopType::Unknown
+    }
+    /// Convert a `DesktopType` into a `String`
+    pub fn to_string(dt:DesktopType)->String {
+    match dt {
+        DesktopType::Application => return String::from("Application"),
+        DesktopType::Link => return String::from("Link"),
+        DesktopType::Directory => return String::from("Directory"),
+        _=> return String::from(""),
+    }
+}
+}
 impl fmt::Display for DesktopType {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "{:?}", self)
@@ -193,7 +227,6 @@ pub fn string_xdg_type(dt:DesktopType)->Option<String> {
 
 /// # Desktop Entry Files
 ///
-/// [XDG specs](https://specifications.freedesktop.org)
 ///
 /// `Option<String>` will be used for all localeString/String
 ///
@@ -206,13 +239,13 @@ pub fn string_xdg_type(dt:DesktopType)->Option<String> {
 ///
 /// `DesktopType` enumerates known "Type" fields and the "Unknown" reserved "Type"
 ///
-/// type is a reserved word in rust, so it is called 'xdg_type'
+/// `type` is a **reserved** word in rust, so it is called 'xdg_type'
 ///
 /// I copied and pasted the website here and made things rustified
 ///
 /// No camelcase inside structs, all lowercase/underscore version of CamelCase
 ///
-/// Comments are just because I didn't delete the rest of the text from the webpage
+/// The text following is from the XDG webpage:
 /// 
 /// # Recognized desktop entry keys
 /// 
@@ -224,56 +257,56 @@ pub fn string_xdg_type(dt:DesktopType)->Option<String> {
 #[derive(Debug)]
 pub struct DesktopEntry {
 //type is a reserved word in rust and other languages so we will use xdg_type
-    /// This specification defines 3 types of desktop entries: Application (type 1), Link (type 2) and Directory (type 3). To allow the addition of new types in the future, implementations should ignore desktop entries with an unknown type.     Stringing    YES
+    /// This specification defines 3 types of desktop entries: Application (type 1), Link (type 2) and Directory (type 3). To allow the addition of new types in the future, implementations should ignore desktop entries with an unknown type.
     pub xdg_type:DesktopType,
-    /// Version of the Desktop Entry Specification that the desktop entry conforms with. Entries that confirm with this version of the specification should use 1.5. Note that the version field is not required to be present.     Stringing    NO    1-3
+    /// Version of the Desktop Entry Specification that the desktop entry conforms with. Entries that confirm with this version of the specification should use 1.5. Note that the version field is not required to be present.
     pub version:f32,
-    /// Specific name of the application, for example "Mozilla".     localeStringing    YES    1-3
+    /// Specific name of the application, for example "Mozilla".
     pub name:Option<String>,
-    /// Generic name of the application, for example "Web Browser".     localeStringing    NO    1-3
+    /// Generic name of the application, for example "Web Browser".
     pub generic_name:Option<String>,
-    /// NoDisplay means "this application exists, but don't display it in the menus". This can be useful to e.g. associate this application with MIME types, so that it gets launched from a file manager (or other apps), without having a menu entry for it (there are tons of good reasons for this, including e.g. the netscape -remote, or kfmclient openURL kind of stuff).     boolean    NO    1-3
+    /// NoDisplay means "this application exists, but don't display it in the menus". This can be useful to e.g. associate this application with MIME types, so that it gets launched from a file manager (or other apps), without having a menu entry for it (there are tons of good reasons for this, including e.g. the netscape -remote, or kfmclient openURL kind of stuff).
     pub no_display:Option<bool>,
-    /// Tooltip for the entry, for example "View sites on the Internet". The value should not be redundant with the values of Name and GenericName.     localeStringing    NO    1-3
+    /// Tooltip for the entry, for example "View sites on the Internet". The value should not be redundant with the values of Name and GenericName.
     pub comment:Option<String>,
-    /// Icon to display in file manager, menus, etc. If the name is an absolute path, the given file will be used. If the name is not an absolute path, the algorithm described in the (Icon Theme Specification)[http://freedesktop.org/wiki/Standards/icon-theme-spec] will be used to locate the icon.     iconStringing    NO    1-3
+    /// Icon to display in file manager, menus, etc. If the name is an absolute path, the given file will be used. If the name is not an absolute path, the algorithm described in the [Icon Theme Specification](https://specifications.freedesktop.org/icon-theme-spec/icon-theme-spec-latest.html#icon_lookup) will be used to locate the icon.
     pub icon:Option<String>,
-    /// Hidden should have been called Deleted. It means the user deleted (at his level) something that was present (at an upper level, e.g. in the system dirs). It's Stringictly equivalent to the .desktop file not existing at all, as far as that user is concerned. This can also be used to "uninstall" existing files (e.g. due to a renaming) - by letting make install install a file with Hidden=true in it.     boolean    NO    1-3
+    /// Hidden should have been called Deleted. It means the user deleted (at his level) something that was present (at an upper level, e.g. in the system dirs). It's Stringictly equivalent to the .desktop file not existing at all, as far as that user is concerned. This can also be used to "uninstall" existing files (e.g. due to a renaming) - by letting make install install a file with Hidden=true in it.
     pub hidden:Option<bool>,
     /// A list of Strings identifying the desktop environments that should display/not display a given desktop entry.
 /// By default, a desktop file should be shown, unless an OnlyShowIn key is present, in which case, the default is for the file not to be shown.
 /// If $XDG_CURRENT_DESKTOP is set then it contains a colon-separated list of Strings. In order, each Stringing is considered. If a matching entry is found in OnlyShowIn then the desktop file is shown. If an entry is found in NotShowIn then the desktop file is not shown. If none of the Strings match then the default action is taken (as above).
 /// $XDG_CURRENT_DESKTOP should have been set by the login manager, according to the value of the DesktopNames found in the session file. The entry in the session file has multiple values separated in the usual way: with a semicolon.
-/// The same desktop name may not appear in both OnlyShowIn and NotShowIn of a group. Stringing(s)    NO    1-3
+/// The same desktop name may not appear in both OnlyShowIn and NotShowIn of a group.
     pub only_show_in:Option<Vec<DesktopEnvironment>>,
     pub not_show_in:Option<Vec<DesktopEnvironment>>,
-    /// A boolean value specifying if D-Bus activation is supported for this application. If this key is missing, the default value is false. If the value is true then implementations should ignore the Exec key and send a D-Bus message to launch the application. (See D-Bus Activation)[https://specifications.freedesktop.org/desktop-entry-spec/latest/ar01s08.html] for more information on how this works. Applications should still include Exec= lines in their desktop files for compatibility with implementations that do not understand the DBusActivatable key.     boolean    NO
+    /// A boolean value specifying if D-Bus activation is supported for this application. If this key is missing, the default value is false. If the value is true then implementations should ignore the Exec key and send a D-Bus message to launch the application. See [D-Bus Activation](https://specifications.freedesktop.org/desktop-entry-spec/latest/ar01s08.html) for more information on how this works. Applications should still include Exec= lines in their desktop files for compatibility with implementations that do not understand the DBusActivatable key.
     pub dbus_activatable:Option<bool>,
-    /// Path to an executable file on disk used to determine if the program is actually installed. If the path is not an absolute path, the file is looked up in the $PATH environment variable. If the file is not present or if it is not executable, the entry may be ignored (not be used in menus, for example).     Stringing    NO    1
+    /// Path to an executable file on disk used to determine if the program is actually installed. If the path is not an absolute path, the file is looked up in the $PATH environment variable. If the file is not present or if it is not executable, the entry may be ignored (not be used in menus, for example).
     pub try_exec:Option<String>,
-    /// Program to execute, possibly with arguments. (See the Exec)[https://specifications.freedesktop.org/desktop-entry-spec/latest/ar01s07.html] key for details on how this key works. The Exec key is required if DBusActivatable is not set to true. Even if DBusActivatable is true, Exec should be specified for compatibility with implementations that do not understand DBusActivatable.     Stringing    NO    1
+    /// Program to execute, possibly with arguments. See the [Exec](https://specifications.freedesktop.org/desktop-entry-spec/latest/ar01s07.html) key for details on how this key works. The Exec key is required if DBusActivatable is not set to true. Even if DBusActivatable is true, Exec should be specified for compatibility with implementations that do not understand DBusActivatable.
     pub exec:Option<String>,
-    /// If entry is of type Application, the working directory to run the program in.     Stringing    NO    1
+    /// If entry is of type Application, the working directory to run the program in.
     pub path:Option<String>,
-    /// Whether the program runs in a terminal window.     boolean    NO    1
+    /// Whether the program runs in a terminal window.
     pub terminal:Option<bool>,
-    /// Identifiers for application actions. This can be used to tell the application to make a specific action, different from the default behavior. (The Application actions section)[https://specifications.freedesktop.org/desktop-entry-spec/latest/ar01s11.html] describes how actions work.     Stringing(s)    NO    1
+    /// Identifiers for application actions. This can be used to tell the application to make a specific action, different from the default behavior. The [Application actions section](https://specifications.freedesktop.org/desktop-entry-spec/latest/ar01s11.html) describes how actions work.
     pub actions:Option<Vec<String>>,
-    /// The MIME type(s) supported by this application.     Stringing(s)    NO    1
+    /// The MIME type(s) supported by this application.
     pub mime_type:Option<Vec<String>>,
-    /// Categories in which the entry should be shown in a menu for possible values (see the Desktop Menu Specification)[http://www.freedesktop.org/Standards/menu-spec].     Stringing(s)    NO    1
+    /// Categories in which the entry should be shown in a menu for possible values see the [Desktop Menu Specification](http://www.freedesktop.org/Standards/menu-spec).
     pub categories:Option<Vec<Categories>>,
-    /// A list of interfaces that this application implements. By default, a desktop file implements no interfaces. (See Interfaces)[https://specifications.freedesktop.org/desktop-entry-spec/latest/ar01s09.html] for more information on how this works.     Stringing(s)    NO
+    /// A list of interfaces that this application implements. By default, a desktop file implements no interfaces. See [Interface]([https://specifications.freedesktop.org/desktop-entry-spec/latest/ar01s09.html) for more information on how this works.
     pub implements:Option<Vec<String>>,
-    /// A list of Strings which may be used in addition to other metadata to describe this entry. This can be useful e.g. to facilitate searching through entries. The values are not meant for display, and should not be redundant with the values of Name or GenericName.     localeStringing(s)    NO    1
+    /// A list of Strings which may be used in addition to other metadata to describe this entry. This can be useful e.g. to facilitate searching through entries. The values are not meant for display, and should not be redundant with the values of Name or GenericName.
     pub keywords:Option<Vec<String>>,
-    /// If true, it is KNOWN that the application will send a "remove" message when started with the DESKTOP_STARTUP_ID environment variable set. If false, it is KNOWN that the application does not work with startup notification at all (does not shown any window, breaks even when using StartupWMClass, etc.). If absent, a reasonable handling is up to implementations (assuming false, using StartupWMClass, etc.). (See the Startup Notification Protocol Specification for more details)[http://www.freedesktop.org/Standards/startup-notification-spec].     boolean    NO    1
+    /// If true, it is KNOWN that the application will send a "remove" message when started with the DESKTOP_STARTUP_ID environment variable set. If false, it is KNOWN that the application does not work with startup notification at all (does not shown any window, breaks even when using StartupWMClass, etc.). If absent, a reasonable handling is up to implementations (assuming false, using StartupWMClass, etc.). See the [Startup Notification Protocol Specification](http://www.freedesktop.org/Standards/startup-notification-spec) for more details.
     pub startup_notify:Option<bool>,
-    /// If specified, it is known that the application will map at least one window with the given Stringing as its WM class or WM name hint (see the Startup Notification Protocol Specification for more details)[http://www.freedesktop.org/Standards/startup-notification-spec].     Stringing    NO    1
+    /// If specified, it is known that the application will map at least one window with the given Stringing as its WM class or WM name hint see the [Startup Notification Protocol Specification](http://www.freedesktop.org/Standards/startup-notification-spec) for more details.
     pub startup_wm_class:Option<String>,
-    /// If entry is Link type, the URL to access.     Stringing    YES    2
+    /// If entry is Link type, the URL to access.
     pub url:Option<String>,
-    /// If true, the application prefers to be run on a more powerful discrete GPU if available, which we describe as “a GPU other than the default one” in this spec to avoid the need to define what a discrete GPU is and in which cases it might be considered more powerful than the default GPU. This key is only a hint and support might not be present depending on the implementation.     boolean    NO    1
+    /// If true, the application prefers to be run on a more powerful discrete GPU if available, which we describe as “a GPU other than the default one” in this spec to avoid the need to define what a discrete GPU is and in which cases it might be considered more powerful than the default GPU. This key is only a hint and support might not be present depending on the implementation.
     pub prefers_non_default_gpu:Option<bool>,
 }
 
